function model = img2model( dimensionality, varargin )
%IMG2MODEL Trains a generative model of protein subcellular location from a
%collection of microscope images.
%
%Inputs
%dimensionality    either '2D' or '3D'
%param             a structure holding possible parameter options
%
%See also IMG2SLML

% Ivan E. Cao-Berg
%
% Copyright (C) 2007-2013 Murphy Lab
% Carnegie Mellon University
%
% ?? ??, 2011 I. Cao-Berg Added 3D model training functionality
% March 23, 2012 I. Cao-Berg Added the creation/deletion of temporary folder
% March 28, 2012 I. Cao-Berg Added a control structure under which if one or more of the 
%                image folders are nonexistent or they do not contain images,
%                the method exits
% March 28, 2012 I. Cao-Berg Added verification of input arguments when training a 2D
%                            generative model
% March 28, 2012 I. Cao-Berg Added verification of input arguments when training a 3D
%                            generative model
% April 10, 2012 I. Cao-Berg Added debug flag to the method. If flag is true, temporary
%                            files will not be deleted
% April 11, 2012 I. Cao-Berg Added verbose flag to the method
% April 17, 2012 I. Cao-Berg Returns an empty model when model cannot be trained
% July 5, 2012 I. Cao-Berg Added training flags to method so that users can train whatever component they wish
% July 26, 2012 Y.Yu Fixed a bug of the order of input argument for ml_traingenmodel2D method
% August 29, 2012 G. Johnson Modified method call to include parameter structure
% May 7, 2013 I. Cao-Berg Included support of masks when training a model
% only for the 2D case
% May 8, 2013 I. Cao-Berg Removed check for the existence of image folder since the check will happen later in the code
% May 15, 2013 I. Cao-Berg Updated method to support wildcards
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu/ or
% send email to murphy@cmu.edu

% Get collection of images
nuclearModelImages = [];
cellModelImages = [];
proteinModelImages = [];

model = [];
switch lower(dimensionality)
    case '2d'
        %icaoberg march 28, 2012
        %check number of input arguments. if there are not 5 input
        %arguments, then the method returns an empty model
        if nargin ~= 5
           warning('CellOrganizer: Wrong number of input arguments.');
           return;
        end

        %check the image directories. if they don't exist or they are
        %empty, the method returns an empty model
        nuclearModelImagesDirectory = varargin{1};

        cellModelImagesDirectory = varargin{2};
 
        proteinModelImagesDirectory = varargin{3};
    
        nuclearModelImages = ml_ls( ...
            nuclearModelImagesDirectory );
        cellModelImages = ml_ls( ...
            cellModelImagesDirectory );
        proteinModelImages = ml_ls( ...
            proteinModelImagesDirectory );
        
        %check that the input parameter is a structure
        param = varargin{4};
        if ~isa( param, 'struct' )
           warning('CellOrganizer: Input argument parameter must be a struct.');
           return;
        end

        %grj 14/5/2013
        debug = setDebugParam(param);
        verbose = setVerboseParam(param);
        
        %icaoberg 7/5/2013
        try
            masks = ml_ls( ...
            param.masks );
        catch
            masks = [];
        end
            
        
        %check existence of temporary folder and make on if it doesn't exist
        if ~exist([pwd filesep 'temp' filesep 'preprocessed'])
          mkdir([pwd filesep 'temp' filesep 'preprocessed']);
        end
        
        % train generative model of protein subcellular location
        param.disp = 'false';
        model = ml_traingenmodel2D( ...
            proteinModelImages,...
            nuclearModelImages, ...
            cellModelImages, ...
            masks, param );
    case '3d'
        %icaoberg march 28, 2012
        %check the existence of the image directory
        dnaImagesDirectoryPath = varargin{1};
        cellImagesDirectoryPath = varargin{2};
        proteinImagesDirectoryPath = varargin{3};
    
        param = varargin{4};
        if ~isa( param , 'struct' )
            warning('CellOrganizer: Input parameter list must be a structure.');
            return
        end
        
        %grj 14/5/2013
        debug = setDebugParam(param);
        verbose = setVerboseParam(param);
        
        %mmackie july 3 2012
        try
            trainFlag = param.train.flag;
            if ~isa( trainFlag, 'char' );
                error('CellOrganizer: training flag must be a string');
            end
        catch
            param.train.flag = 'all';
            trainFlag = param.train.flag;
        end
        
        %icaoberg july, 5 2012
        if ~strcmpi( trainFlag, 'nuclear' ) && ...
           ~strcmpi( trainFlag, 'framework' ) && ...
           ~strcmpi( trainFlag, 'all' )
            error('CellOrganizer: Unrecognized training flag');
        end

        %check existence of temporary folder and make on if it doesn't exist
        if ~exist( [ pwd filesep 'temp'], 'dir' )
            mkdir( [ pwd filesep 'temp'] );
        end
        
        try
            %icaoberg april 17, 2012
            model.dimensionality = '3D';
            
            if verbose
               %clc; 
               fprintf( 1, '%s\n', 'Training nuclear shape model' );
            end
                
            %gj aug 29, 2012
            %passes in 'param' now
            model.nuclearShapeModel = train_nuc_shape_model( ...
                dnaImagesDirectoryPath, ...
                cellImagesDirectoryPath, ...
                proteinImagesDirectoryPath, ...
                param );

            %mmackie july 3, 2012
            if strcmpi( trainFlag, 'framework' ) || strcmpi( trainFlag, 'all' )
                fprintf( 1, '%s\n', 'Training cell shape model' );
                model.cellShapeModel = train_cell_shape_model2( ...
                    dnaImagesDirectoryPath, ...
                    cellImagesDirectoryPath, ...
                    proteinImagesDirectoryPath, ...
                    param );
            end

            if strcmpi( trainFlag, 'all' )
                if verbose
                  %clc; 
                  fprintf( 1, '%s\n', 'Training protein model' );
                end
                %D. Sullivan 2/22/13 added param structure to pass resolution
                model.proteinShape = train_protein_model( ...
                    dnaImagesDirectoryPath, ...
                    cellImagesDirectoryPath, ...
                    proteinImagesDirectoryPath, ...
                    param );
            end

           %icaoberg 22/02/2013
           model.info.original_resolution = param.model.original_resolution;
           model.info.downsampling_vector = param.model.downsampling;
           model.nuclearShapeModel = struct('resoluton', param.model.resolution);
           model.cellShapeModel.resolution = param.model.resolution;
           %D. Sullivan 2/24/2013 This should be set already in
           %train_protein_model
           %model.proteinShape.resolution = param.model.protein_resolution;

        catch err
           %icaoberg april 17, 2012
           %returns empty model if model cannot be trained
                     
           model = [];
           warning('CellOrganizer: Unable to train 3D generative model.');
          
           %icaoberg 06/02/2013
           if debug
               getReport( err, 'extended' )
           end
           
           if ~debug
              rmdir( 'temp', 's' );
           end
           
           return
        end
    otherwise
        error(['CellOrganizer: Unknown dimensionality ' ...
            dimensionality]);
end

if ~debug
   rmdir( 'temp', 's' );
else
   if verbose
       fprintf(1, '%s\n', ...
         'CellOrganizer: Debug mode is on, temporary files will not be deleted.');
   end
end

end%img2model

function [debug] = setDebugParam(param)
%set debug default to true
    try
        debug = param.debug;
        if ischar(debug)
            if strcmpi(debug, 'false')
               debug = false;
            else 
               debug = true;
            end
        else
            debug = logical(debug);
        end
    catch
        debug = true;
    end 
end

function [verbose] = setVerboseParam(param)
%set verbose default to true
    try
        verbose = param.verbose;
        if ischar(verbose)
            if strcmpi(verbose, 'false')
               verbose = false;
            else 
               verbose = true;
            end
        else
            verbose = logical(verbose);
        end
    catch
        verbose = true;
    end 
end
